#include "util/Common.h"
#include "Logger.h"
#include "messenger/MQTTLogger.h"

namespace Example {

class PahoLogger :
        public Utility::ILogListener {
public:

    /**
     * @brief Default logger level for logger from paho library
     */
    static const Example::Logger::LoggerLevel DEFAULT_LOGGER_LEVEL;

    /**
     * @fn PahoLogger();
     * @brief Contrustor.
     */
    PahoLogger();

    /**
     * @fn virtual ~PahoLogger();
     * @brief Destructor.
     */
     virtual ~PahoLogger();

    /**
     * @fn  void OnTrace(const std::string& _msg);
     * @brief Used to report trace messages. Trace messages have the lowest log level.
     * @param _msg Log message.
     */
     void OnTrace(const std::string& _msg);

    /**
     * @fn  void OnDebug(const std::string& _msg);
     * @brief Used to report debug messages.
     * @param _msg Log message.
     */
     void OnDebug(const std::string& _msg);

    /**
     * @fn  void OnInfo(const std::string& _msg);
     * @brief Used to report information messages.
     * @param _msg Log message.
     */
     void OnInfo(const std::string& _msg);

    /**
     * @fn  void OnWarn(const std::string& _msg);
     * @brief Used to report warning messages.
     * @param _msg Log message.
     */
     void OnWarn(const std::string& _msg);

    /**
     * @fn  void OnError(const std::string& _msg);
     * @brief Used to report error messages.
     * @param _msg Log message.
     */
     void OnError(const std::string& _msg);

     /**
      * @fn  void OnFatal(const std::string& _msg);
      * @brief Used to report fatal messages.
      * @param _msg Log message.
      */
      void OnFatal(const std::string& _msg);
private:
     Logger m_Logger;
};

}
