#include "Config.h"

using namespace Utility;

const std::string Config::DELIMITER = " = ";

Config::Config(const std::string &_fileName) {
    m_FileName = _fileName;
}

bool Config::HasElements() const {
    return !m_ConfigMap.empty();
}

std::string Config::ExtractStringValue(const std::string &_key, const std::string& _def) const {
    ConfigMap::const_iterator itr = m_ConfigMap.find(_key);
    if (itr != m_ConfigMap.end()) {
        return itr->second;
    } else {
        return _def;
    }
}

int Config::ExtractIntValue(const std::string &_key, int _def) const {
    ConfigMap::const_iterator itr = m_ConfigMap.find(_key);
    if (itr != m_ConfigMap.end()) {
        float ret = _def;
        std::istringstream(itr->second) >> ret;
        return ret;
    } else {
        return _def;
    }
}

float Config::ExtractFloatValue(const std::string &_key, float _def) const {
    ConfigMap::const_iterator itr = m_ConfigMap.find(_key);
    if (itr != m_ConfigMap.end()) {
        float ret = _def;
        std::istringstream(itr->second) >> ret;
        return ret;
    } else {
        return _def;
    }
}

bool Config::ExtractBoolValue(const std::string &_key, bool _def) const {
    ConfigMap::const_iterator itr = m_ConfigMap.find(_key);
    if (itr != m_ConfigMap.end()) {
        bool ret = _def;
        std::istringstream(itr->second) >> std::boolalpha >> ret;
        return ret;
    } else {
        return _def;
    }
}

void Config::InsertStringValue(const std::string &_key, const std::string &_value) {
    std::ofstream ofstr(m_FileName.c_str(), std::ios_base::out | std::ios_base::trunc);
    m_ConfigMap[_key] = _value;

    WriteAll(ofstr);

    ofstr.close();
}

void Config::CleanAll() {
    m_ConfigMap.clear();
    std::ofstream ofstr(m_FileName.c_str(), std::ios_base::out | std::ios_base::trunc);
    ofstr.close();
}

void Config::Parse() {
    std::string line;
    std::ifstream ifstr(m_FileName.c_str());
    if (ifstr.good()) {
        while(!std::getline(ifstr, line).eof()) {
            if (!line.empty() && line.at(0) != '#') {
                std::string first = line.substr(0, line.find(DELIMITER));
                std::string second = line.substr(line.find(DELIMITER) + DELIMITER.length());
                m_ConfigMap[first] = second;
            }
        }
    }
    ifstr.close();
}

void Config::WriteAll(std::ofstream &_ofstr) const {
    ConfigMap::const_iterator itr = m_ConfigMap.begin();
    for (; itr != m_ConfigMap.end(); ++itr) {
        _ofstr << itr->first + DELIMITER + itr->second << std::endl;
    }
}
