#include "Logger.h"

using namespace Example;

void LoggerHelper::Initialize(Logger::LoggerLevel _globalLevel, const std::string& _fileName, bool _immediate, int _maxRoll, unsigned long _maxFileSize) {
    humble::logging::Factory &fac = humble::logging::Factory::getInstance();
    fac.setDefaultFormatter(new LogFormatter());
    int logLevel = _globalLevel;
    fac.setConfiguration(new humble::logging::SimpleConfiguration(logLevel));
    fac.registerAppender(new FlushConsoleAppender());
    fac.registerAppender(new humble::logging::RollingFileAppender(_fileName, _immediate, _maxRoll, _maxFileSize));
}

Logger::Logger(const std::string &_name, LoggerLevel _logLevel) :
    m_Name(_name),
    m_LogLevel(_logLevel) {

    m_Logger = &(humble::logging::Factory::getInstance().getLogger(m_Name));
    m_Logger->setLogLevel(_logLevel);
}

Logger::~Logger() {
}

Logger::LoggerLevel StringToLogLevel(const std::string& _logLevel) {
    if (_logLevel == "D") {
        return Logger::DEBUG;
    } else if (_logLevel == "I") {
        return Logger::INFO;
    } else if (_logLevel == "W") {
        return Logger::WARN;
    } else if (_logLevel == "E") {
        return Logger::ERROR;
    } else if (_logLevel == "F") {
        return Logger::FATAL;
    } else if (_logLevel == "N") {
        return Logger::OFF;
    } else {
        return Logger::TRACE;
    }
}

void Logger::SetLogLevel(const std::string& _logLevel) {
    LoggerLevel logLevel = StringToLogLevel(_logLevel);
    m_Logger->setLogLevel(logLevel);
}

void Logger::SetLogLevel(LoggerLevel _logLevel) {
    m_Logger->setLogLevel(_logLevel);
}

void Logger::Fatal(const std::string& _msg) {
    HUMBLE_LOG_NAME((*m_Logger), _msg, humble::logging::LogLevel::Fatal, m_Name);
}

void Logger::Error(const std::string& _msg) {
    HUMBLE_LOG_NAME((*m_Logger), _msg, humble::logging::LogLevel::Error, m_Name);
}

void Logger::Warn(const std::string& _msg) {
    HUMBLE_LOG_NAME((*m_Logger), _msg, humble::logging::LogLevel::Warn, m_Name);
}

void Logger::Info(const std::string& _msg) {
    HUMBLE_LOG_NAME((*m_Logger), _msg, humble::logging::LogLevel::Info, m_Name);
}

void Logger::Debug(const std::string& _msg) {
    HUMBLE_LOG_NAME((*m_Logger), _msg, humble::logging::LogLevel::Debug, m_Name);
}

void Logger::Trace(const std::string& _msg) {
    HUMBLE_LOG_NAME((*m_Logger), _msg, humble::logging::LogLevel::Trace, m_Name);
}

LogFormatter::LogFormatter() {
}

LogFormatter::LogFormatter(const LogFormatter& _other) {
}

LogFormatter::~LogFormatter() {
}

humble::logging::Formatter* LogFormatter::copy() const {
    return new LogFormatter(*this);
}

std::string LogFormatter::format(const humble::logging::LogEvent &_logEvent) const {
    const LogEventWithName& logEvent = static_cast<const LogEventWithName&>(_logEvent);

    std::string lls = humble::logging::LogLevel::resolveLogLevel(logEvent.getLogLevel());

    std::stringstream ssRelativeTid;
    ssRelativeTid << logEvent.getTid() - logEvent.getPid();
    std::stringstream ssPid;
    ssPid << logEvent.getPid();

    struct timeval timeNow;
    gettimeofday(&timeNow, NULL);
    struct tm timeInfo;
    localtime_r(&timeNow.tv_sec, &timeInfo);
    char timeString [80];
    std::strftime(timeString, 80, "%Y-%m-%d %H:%M:%S", &timeInfo);
    char currentTime[84] = "";
    sprintf(currentTime, "%s.%d", timeString, int (timeNow.tv_usec / 1000));

    char logLevel = toupper(lls[0]);
    std::stringstream ss;
    ss << "[" << std::string(currentTime) << "]"
       << " " << logLevel << " "
       << "[" << ssPid.str() << "] "
       << "[" << ssRelativeTid.str() << "] "
       << "[" << logEvent.GetName() << "] "
       << logEvent.getMessage()
       << "\r\n";

    return ss.str();
}

FlushConsoleAppender::FlushConsoleAppender() {
}

FlushConsoleAppender::~FlushConsoleAppender() {
}

void FlushConsoleAppender::log(const humble::logging::LogEvent &_logEvent) {
    ConsoleAppender::log(_logEvent);
    std::cout.flush();
}

LogEventWithName::LogEventWithName(int level, const std::string &message, int line, const std::string &file, const std::string &func, const std::string& _name) :
    humble::logging::LogEvent(level, message, line, file, func),
    m_Name(_name) {
}

const std::string& LogEventWithName::GetName() const {
    return m_Name;
}
