#include <iostream>

#include "RPIDemoDevice.h"
#include "Config.h"
#include "Logger.h"
#include <signal.h>

using namespace SSDP;
using namespace Messenger;
using namespace IPDevice;
using namespace Example;
using namespace Utility;

static const Example::Logger::LoggerLevel DEFAULT_GLOBAL_LOGGER_LEVEL = Example::Logger::INFO;
static const std::string DEFAULT_LOG_FILE = "Log.txt";
static const std::string DEFAULT_DATA_FILE = "Data.txt";

Example::Logger rootLogger("Application", DEFAULT_GLOBAL_LOGGER_LEVEL);

std::string makePrettyText(std::string text, char symbol, unsigned int lineSize) {
    int cnt = (lineSize - text.length())/2;
    std::string symbols(cnt, symbol);
    std::string line = symbols + text + symbols;

    if ( line.length()%2 ) {
        line += symbol;
    }

    return line;
}

void printLogHeader() {
    std::stringstream ss;
    ss<< makePrettyText("", '-', 80);

    rootLogger.Info(ss.str());
    rootLogger.Info(" WISE ");
    rootLogger.Info(" Platform: " + std::string(Device::GetPlatform()));
    rootLogger.Info(" Type: " + std::string(Device::GetType()));
    rootLogger.Info(" Version: " + std::string(Device::GetVersion()));
    rootLogger.Info(" Build time: " + std::string(__DATE__) + " " + std::string(__TIME__));
    rootLogger.Info(makePrettyText("", '-', 80));
}

void WaitForFinish() {
    sigset_t sset;
    sigemptyset(&sset);
    sigaddset(&sset, SIGINT);
    sigaddset(&sset, SIGQUIT);
    sigaddset(&sset, SIGTERM);
    sigprocmask(SIG_BLOCK, &sset, NULL);
    int sig;
    sigwait(&sset, &sig);

    rootLogger.Info("Shutting down application");
}

int main(int argc, char* argv[]) {
    Example::LoggerHelper::Initialize(DEFAULT_GLOBAL_LOGGER_LEVEL, DEFAULT_LOG_FILE, true, 3, 1024L * 1024L);

    printLogHeader();

    if (argc != 2) {
        rootLogger.Fatal("Application requires 1 argument:");
        rootLogger.Fatal("1. Path to config file");
    } else {
        try {
            Config config(argv[1]);
            config.Parse();
            rootLogger.SetLogLevel(config.ExtractStringValue("LogLevel.Main"));

            Config data(DEFAULT_DATA_FILE);
            data.Parse();

            DeviceInformation devInfo;
            std::string name = data.ExtractStringValue("Name");
            if (name.empty()) {
                name = config.ExtractStringValue("Device.Name");
            }
            devInfo.SetName(name);
            devInfo.SetManufacturer(config.ExtractStringValue("Device.Manufacturer"));
            devInfo.SetModel(config.ExtractStringValue("Device.Model"));
            devInfo.SetVersion(config.ExtractStringValue("Device.Version"));
            std::string identifier = config.ExtractStringValue("Device.Identifier");
            devInfo.SetIdentifier(identifier);
            devInfo.SetVersionNumber(config.ExtractIntValue("Device.VersionNumber"));
            devInfo.SetProtocolVersion(config.ExtractIntValue("Device.ProtocolVersion"));
            devInfo.SetMAC(identifier.substr(0, identifier.find('.')));

            RPIDemoDevice device;

            device.SetData(data);
            device.SetConfig(config);

            device.SetDeviceInfomation(devInfo);

            //device.SetDescProviderUrl(config.ExtractValue("Inclusion.DescProviderUrl"));

            device.Start();

            WaitForFinish();
        } catch (const BaseException& _exc) {
            std::stringstream ss;
            ss << _exc.GetMessage() << " <c=" << _exc.GetCode() << ">";
            rootLogger.Fatal(ss.str());
            /* TODO: Retry */
        }
        return 0;
    }
}
