/****************************************************************************
 *  Copyright (c) 2018 Oblo Living d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.obloliving.com/
 *  -----------------------------------------------------
 *
 * \project Oblo WISE SDK - SDK library
 * \file MQTTLogger.h
 * \brief
 * PAHO logger abstraction classes.
 *
 * @Author Igor Stefanovic
 *
 * \notes
 *
 * \history
 *****************************************************************************/
#ifndef MQTT_LOGGER_H
#define MQTT_LOGGER_H

#include "util/Common.h"
#include "util/Logger.h"

extern "C" {
    #include "paho/Log.h"
}

namespace Messenger {

/**
 * @class Logger
 * @brief Logger class for Paho library
 */
class DLLExport MQTTLogger {
public:
    /**
     * @fn void SetListener(Utility::ILogListener* _listener);
     * @brief Sets log listener for paho library logs.
     * @param _listener Pointer to log listener.
     */
    void SetListener(Utility::ILogListener* _listener);

    /**
     * @fn static Logger& GetInstance();
     * @brief Provides instance of paho logger.
     * @return Reference to Logger class.
     */
    static MQTTLogger& GetInstance();

    /**
     * @fn static void LogCallback(enum LOG_LEVELS _level, const char* _message);
     * @brief Callback for paho library logger, which must be registered to library.
     * @param _level Paho log level.
     * @param _message Paho log message.
     */
    static void LogCallback(enum LOG_LEVELS _level, const char* _message);

    /**
     * @fn void OnTrace(const std::string& _msg);
     * @brief Handles trace messages.
     * @param _msg Message.
     */
    void OnTrace(const std::string& _msg);

    /**
     * @fn void OnDebug(const std::string& _msg);
     * @brief Handles debug messages.
     * @param _msg Message.
     */
    void OnDebug(const std::string& _msg);

    /**
     * @fn void OnInfo(const std::string& _msg);
     * @brief Handles information messages.
     * @param _msg Message.
     */
    void OnInfo(const std::string& _msg);

    /**
     * @fn void OnWarn(const std::string& _msg);
     * @brief Handles warning messages.
     * @param _msg Message.
     */
    void OnWarn(const std::string& _msg);

    /**
     * @fn void OnError(const std::string& _msg);
     * @brief Handles error messages.
     * @param _msg Message.
     */
    void OnError(const std::string& _msg);

    /**
     * @fn void OnFatal(const std::string& _msg);
     * @brief Handles fatal messages.
     * @param _msg Message.
     */
    void OnFatal(const std::string& _msg);
private:
    MQTTLogger();
    ~MQTTLogger();

    MQTTLogger(const MQTTLogger& _logger);
    void operator =(const MQTTLogger& _logger);

    Utility::ILogListener* m_Listener;
};

}

#endif // MQTT_LOGGER_H
