/****************************************************************************
 *  Copyright (c) 2018 Oblo Living d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.obloliving.com/
 *  -----------------------------------------------------
 *
 * \project Oblo WISE SDK - SDK library
 * \file MessageHolder.h
 * \brief
 * MQTT Message holder classes.
 *
 * @Author Igor Stefanovic
 *
 * \notes
 *
 * \history
 *****************************************************************************/
#ifndef MESSAGE_HOLDER_H
#define MESSAGE_HOLDER_H

#include "util/Common.h"
#include "util/SharedPointer.h"
#include "messenger/MQTTMessage.h"

/**
 * @namespace Messenger
 * @brief Implementation of MQTT Client.
 */
namespace Messenger {

/**
 * @class PublishHolder
 * @brief Publish message holder.
 */
class DLLExport PublishHolder {
public:
    /**
     * @typedef Utility::SharedPointer<PublishHolder> Ptr;
     * @brief Shared pointer to PublishHolder object.
     */
    typedef Utility::SharedPointer<PublishHolder> Ptr;

    /**
     * @fn PublishHolder(const std::string& _topic, const Message::Ptr _msg);
     * @brief Constructor.
     * @param _topic Topic.
     * @param _msg MQTT message.
     */
    PublishHolder(const std::string& _topic, const Message::Ptr _msg);

    /**
     * @fn virtual ~PublishHolder();
     * @brief Destructor.
     */
    virtual ~PublishHolder() {}

    /**
     * @fn const std::string GetTopic() const;
     * @brief Getter for topic.
     * @return Topic.
     */
    const std::string GetTopic() const;

    /**
     * @fn const Message::Ptr GetMessage() const;
     * @brief Getter for message.
     * @return Shared pointer to message.
     */
    const Message::Ptr GetMessage() const;
private:
    const std::string m_Topic;
    const Message::Ptr m_Message;
};

/**
 * @class RecvHolder
 * @brief Holder for a received message.
 */
class DLLExport RecvHolder {
public:
    /**
     * @typedef Utility::SharedPointer<PublishHolder> Ptr;
     * @brief Shared pointer to RecvHolder object.
     */
    typedef Utility::SharedPointer<RecvHolder> Ptr;

    /**
     * @fn RecvHolder (const std::string& _topic, const Message::Ptr _msg);
     * @brief Constructor.
     * @param _topic Topic.
     * @param _msg MQTT message.
     */
    RecvHolder (const std::string& _topic, const Message::Ptr _msg);

    /**
     * @fn virtual ~RecvHolder();
     * @brief Destructor.
     */
    virtual ~RecvHolder() {}

    /**
     * @fn const std::string GetTopic() const;
     * @brief Getter for topic.
     * @return Topic.
     */
    const std::string GetTopic() const;

    /**
     * @fn const Message::Ptr GetMessage() const;
     * @brief Getter for message.
     * @return Shared pointer to message.
     */
    const Message::Ptr GetMessage() const;
private:
    const std::string m_Topic;
    const Message::Ptr m_Message;
};

}


#endif // MESSAGE_HOLDER_H
