/****************************************************************************
 *  Copyright (c) 2018 Oblo Living d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.obloliving.com/
 *  -----------------------------------------------------
 *
 * \project Oblo WISE SDK - SDK library
 * \file SocketAddress.h
 * \brief
 * Socket address abstraction class.
 *
 * @Author Igor Stefanovic
 *
 * \notes
 *
 * \history
 *****************************************************************************/
#ifndef SOCKET_ADDRESS_H
#define SOCKET_ADDRESS_H

#include "util/Common.h"
#include <arpa/inet.h>

/**
 * @namespace SSDP
 * @brief SSDP client implementation
 */
namespace SSDP {

/**
 * @class SocketAddress
 * @brief Abstraction class for socket address.
 */
class DLLExport SocketAddress {
public:
    /**
     * @fn SocketAddress();
     * @brief Constructor.
     */
    SocketAddress();

    /**
     * @fn SocketAddress(const std::string& _addr, unsigned int _port);
     * @brief Constructor.
     * @param _addr IP address.
     * @param _port Port.
     */
    SocketAddress(const std::string& _addr, unsigned int _port);

    /**
     * @fn SocketAddress(const struct sockaddr_in* _sockAddr);
     * @brief Constructor.
     * @param _sockAddr Struct of socket address.
     */
    SocketAddress(const struct sockaddr_in* _sockAddr);

    /**
     * @fn struct sockaddr* GetSocketAddress() const;
     * @brief GetSocketAddress
     * @return
     */
    struct sockaddr* GetSocketAddress() const;

    /**
     * @fn in_port_t GetPort() const;
     * @brief Getter for Port field.
     * @return Current Port.
     */
    in_port_t GetPort() const;

    /**
     * @fn struct in_addr GetAddress() const;
     * @brief Getter for IPAddress field.
     * @return Current IPAdress.
     */
    struct in_addr GetAddress() const;

    /**
     * @fn unsigned int GetSize() const;
     * @brief Getter for size of socket address struct.
     * @return Size of socket address struct.
     */
    unsigned int GetSize() const;
private:
    struct sockaddr_in m_SocketAddress;
};
}


#endif // SOCKET_ADDRESS_H
