/****************************************************************************
 *  Copyright (c) 2018 Oblo Living d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.obloliving.com/
 *  -----------------------------------------------------
 *
 * \project Oblo WISE SDK - SDK library
 * \file Common.h
 * \brief
 * Utilily functions and includes.
 *
 * @Author Igor Stefanovic
 *
 * \notes
 *
 * \history
 *****************************************************************************/
#ifndef COMMON_H
#define COMMON_H

#include <string>
#include <sstream>
#include <cstring>
#include <iostream>
#include <list>
#include <map>
#include <vector>
#include <queue>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <pthread.h>
#include "util/ErrorCodes.h"

#if defined(WIN32) || defined(WIN64)
    #define DLLImport __declspec(dllimport)
    #define DLLExport __declspec(dllexport)
#else
    #define DLLImport extern
    #define DLLExport __attribute__ ((visibility ("default")))
#endif

#define SAFE_DELETE_O(p) delete p; p = NULL;
#define SAFE_DELETE_ARRAY_O(a) delete [] a; a = NULL;

/**
 * @namespace Utility
 * @brief Abstraction utility classes.
 */
namespace Utility {

/**
 * @fn char* DuplicateString(const std::string& _szBuff);
 * @brief Duplicates string.
 * @param _szBuff String to be duplicated
 * @return New string.
 */
DLLExport char* DuplicateString(const std::string& _szBuff);

/**
 * @fn int StoI(const std::string& _str);
 * @brief Converts string to integer.
 * @param _str String to be converted.
 * @return Number after conversion.
 */
DLLExport int StoI(const std::string& _str);

/**
 * @fn std::string ItoS(unsigned int _int);
 * @brief Converting integer to string.
 * @param _int Integer to be converted.
 * @return String after conversion.
 */
DLLExport std::string ItoS(unsigned int _int);
}

#endif // COMMON_H
