/****************************************************************************
 *  Copyright (c) 2018 Oblo Living d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.obloliving.com/
 *  -----------------------------------------------------
 *
 * \project Oblo WISE SDK - SDK library
 * \file Event.h
 * \brief
 * Utility Event abstraction class.
 *
 * @Author Igor Stefanovic
 *
 * \notes
 *
 * \history
 *****************************************************************************/
#ifndef EVENT_H
#define EVENT_H

#include <sys/time.h>
#include <time.h>
#include "util/Common.h"
#include "util/Exceptions.h"

/**
 * @namespace Utility
 * @brief Abstraction utility classes.
 */
namespace Utility {

/**
 * @class Event
 * @brief Implementation class for Event syncronization functionality.
 */
class DLLExport Event {
public:
    /**
     * @fn Event();
     * @brief Constructor.
     * @throws Utility::BaseException
     */
    Event();

    /**
     * @fn ~Event();
     * @brief Destructor.
     */
    ~Event();

    /**
     * @fn void Set();
     * @brief Setting the event.
     * @throws Utility::BaseException
     */
    void Set();

    /**
     * @fn void Wait();
     * @brief Waiting on condition variable to be reset.
     * @throws Utility::BaseException
     */
    void Wait();

    /**
     * @fn bool Wait(long _milliseconds);
     * @brief Waiting on condition reset or the time to expire defined by milliseconds parameter.
     * @param _milliseconds Time to wait.
     * @return Indication whether the waiting is successful.
     * @throws Utility::TimeoutException
     */
    bool Wait(long _milliseconds);

    /**
     * @fn void Reset();
     * @brief Resets the condition variable.
     * @throws Utility::TimeoutException
     */
    void Reset();
private:
    bool m_State;
    pthread_mutex_t m_Mutex;
    pthread_cond_t m_Cond;
};

}

#endif // EVENT_H
