/****************************************************************************
 *  Copyright (c) 2018 Oblo Living d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.obloliving.com/
 *  -----------------------------------------------------
 *
 * \project Oblo WISE SDK - SDK library
 * \file Mutex.h
 * \brief
 * Utility Mutex abstraction class.
 *
 * @Author Igor Stefanovic
 *
 * \notes
 *
 * \history
 *****************************************************************************/
#ifndef MUTEX_H
#define MUTEX_H

#include <sys/time.h>
#include "util/Common.h"

/**
 * @namespace Utility
 * @brief Abstraction utility classes.
 */
namespace Utility {

/**
 * @class Mutex
 * @brief Mutex abstraction class.
 */
class DLLExport Mutex {
public:
    /**
     * @fn Mutex();
     * @brief Constructor.
     */
    Mutex();

    /**
     * @fn ~Mutex();
     * @brief Destructor.
     */
    ~Mutex();

    /**
     * @fn void Lock();
     * @brief Locking the mutex.
     */
    void Lock();

    /**
     * @fn bool TryLock();
     * @brief Tries to lock the mutex.
     * @return Indication whether the lock was successful.
     */
    bool TryLock();

    /**
     * @fn bool TryLock(long milliseconds);
     * @brief Tries to lock the mutex for time defined by parameter.
     * @param milliseconds Time for mutex to be locked.
     * @return Indication whether the lock was successful.
     */
    bool TryLock(long milliseconds);

    /**
     * @fn void Unlock();
     * @brief Unlocking mutex.
     */
    void Unlock();
private:
    pthread_mutex_t m_Mutex;
};

}

#endif // MUTEX_H
