/****************************************************************************
 *  Copyright (c) 2018 Oblo Living d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.obloliving.com/
 *  -----------------------------------------------------
 *
 * \project Oblo WISE SDK - SDK library
 * \file ScopedLock.h
 * \brief
 * Utility Scoped lock class.
 *
 * @Author Igor Stefanovic
 *
 * \notes
 *
 * \history
 *****************************************************************************/
#ifndef SCOPED_LOCK_H
#define SCOPED_LOCK_H

#include "Mutex.h"

/**
 * @namespace Utility
 * @brief Abstraction utility classes.
 */
namespace Utility {

/**
 * @class ScopedLock
 * @brief Scoped lock implementation class
 */
class DLLExport ScopedLock {
public:
    /**
     * @fn ScopedLock(Mutex& mutex);
     * @brief Constructor which locks the mutex.
     * @param mutex Mutex.
     */
    ScopedLock(Mutex& mutex);

    /**
     * @fn ~ScopedLock();
     * @brief Destructor which unlocks the mutex.
     */
    ~ScopedLock();
private:
    Mutex& m_Mutex;
};
}

#endif // SCOPED_LOCK_H
