/****************************************************************************
 *  Copyright (c) 2018 Oblo Living d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.obloliving.com/
 *  -----------------------------------------------------
 *
 * \project Oblo WISE SDK - SDK library
 * \file SynchronizedObject.h
 * \brief
 * Utility Synchronized object class.
 *
 * @Author Igor Stefanovic
 *
 * \notes
 *
 * \history
 *****************************************************************************/
#ifndef SYNCHRONIZED_OBJECT_H
#define SYNCHRONIZED_OBJECT_H

#include "Mutex.h"
#include "Event.h"

/**
 * @namespace Utility
 * @brief Abstraction utility classes.
 */
namespace Utility {

/**
 * @class SynchronizedObject
 * @brief Implemenentation of synchronized object.
 */
class DLLExport SynchronizedObject {
public:
    /**
     * @fn SynchronizedObject();
     * @brief Constructor.
     */
    SynchronizedObject() {}

    /**
     * @fn virtual ~SynchronizedObject();
     * @brief Destructor.
     */
    virtual ~SynchronizedObject() {}

    /**
     * @fn void Lock();
     * @brief Lock the synchronized object.
     */
    void Lock();

    /**
     * @fn bool TryLock();
     * @brief Tries to lock the synchronized object.
     * @return Indication whether to locking was successful.
     */
    bool TryLock();

    /**
     * @fn void Unlock();
     * @brief Unlock the synchronized object.
     */
    void Unlock();

    /**
     * @fn void Notify();
     * @brief Notifies the waiting synchronized object.
     */
    void Notify();

    /**
     * @fn void Wait();
     * @brief Waiting on condition reset.
     */
    void Wait();

    /**
     * @fn void Wait(long milliseconds);
     * @brief Waiting on condition reset or the time to expire defined by milliseconds parameter.
     * @param milliseconds Time to wait.
     */
    void Wait(long milliseconds);

    /**
     * @fn bool TryWait(long milliseconds);
     * @brief TryWait
     * @param milliseconds
     * @return Indication whether the waiting is successful.
     */
    bool TryWait(long milliseconds);
private:
    Mutex m_Mutex;
    Event m_Event;
};
}

#endif // SYNCHRONIZED_OBJECT_H
