#include "ipdevice/Command.h"

using namespace IPDevice;
using namespace Utility;

InputParam::InputParam(const std::string &_name, InputParamType _type) : m_Name(_name), m_Type(_type) {
}

void InputParam::SetName(const std::string &_name) {
    m_Name = _name;
}

void InputParam::SetType(InputParamType _type) {
    m_Type = _type;
}

InputParam& InputParam::AddOption(Jzon::Node _option) {
    m_Options.push_back(_option);
    return *this;
}

InputParam& InputParam::AddRange(Jzon::Node _min, Jzon::Node _max) {
    m_Options.push_back(_min);
    m_Options.push_back(_max);
    return *this;
}

const std::vector<Jzon::Node>& InputParam::GetOptions() const {
    return m_Options;
}

std::string InputParam::GetName() const {
    return m_Name;
}

InputParam::InputParamType InputParam::GetType() const {
    return m_Type;
}

Jzon::Node InputParam::GetMin() const {
    if (m_Type == RANGE) {
        return m_Options[0];
    } else {
        return 0;
    }
}

Jzon::Node InputParam::GetMax() const {
    if (m_Type == RANGE) {
        return m_Options[1];
    } else {
        return 0;
    }
}

void InputParam::ToJson(JsonStruct &_json) const {
    _json.Insert("name", m_Name);
    _json.Insert("type", m_Type);
    if (m_Type == RANGE) {
        _json.Insert("range", m_Options);
    } else if (m_Type == ATTRIBUTE) {
        _json.Insert("options", m_Options);
    }
}

Command::Command(const std::string _name, ServiceCommandDelegate::Ptr _function) : m_Name(_name), m_Function(_function), m_HasParamAny(false) {

}

std::string Command::GetName() const {
    return m_Name;
}

void Command::SetName(const std::string &_name) {
    m_Name = _name;
}

Command& Command::AddInputParam(const InputParam &_inputParam) {
    if (_inputParam.GetType() == InputParam::ANY && !m_HasParamAny) {
        m_HasParamAny = true;
        m_InputParams.push_back(_inputParam);
    } else if (_inputParam.GetType() != InputParam::ANY) {
        m_InputParams.push_back(_inputParam);
    }
    return *this;
}

void Command::SetCommandFunction(ServiceCommandDelegate::Ptr _function) {
    m_Function = _function;
}

ServiceCommandDelegate::Ptr Command::GetCommandFunction() const {
    return m_Function;
}

bool Command::HasParamAny() const {
    return m_HasParamAny;
}

const std::vector<InputParam>& Command::GetInputParams() const {
    return m_InputParams;
}

void Command::ToJson(JsonStruct &_json) const {
    _json.Insert("name", m_Name);
    std::vector<InputParam>::const_iterator itr = m_InputParams.begin();
    std::vector<Jzon::Node> vector;
    for (; itr != m_InputParams.end(); ++itr) {
        JsonStruct json;
        itr->ToJson(json);
        vector.push_back(json.GetNode());
    }
    _json.Insert("input", vector);
}
