#include "ipdevice/Group.h"

using namespace IPDevice;
using namespace Utility;

Group::Group(unsigned int _id) : m_Id(_id) {
}

Group::~Group() {
    std::map<std::string, BaseService*>::iterator itr = m_ServiceList.begin();
    for (; itr != m_ServiceList.end(); ++itr) {
        SAFE_DELETE_O(itr->second)
    }
}

void Group::AddService(BaseService *_baseService) {
    m_ServiceList[_baseService->GetServiceType()] = _baseService;
}

void Group::ToJson(JsonStruct &_json, unsigned long long _timestamp, bool _stripped) const {
    std::vector<Jzon::Node> array;
    ConstIterator itr = m_ServiceList.begin();
    for (; itr != m_ServiceList.end(); ++itr) {
        JsonStruct json;
        itr->second->ToJson(json, _timestamp, _stripped);
        if (!json.IsEmpty()) {
            array.push_back(json.GetNode());
        }
    }
    if (!array.empty()) {
        _json.Insert("serviceList", array);
        _json.Insert("id", m_Id);
    }
}

unsigned int Group::GetGroupId() const {
    return m_Id;
}

BaseService* Group::FindServiceByType(const std::string &_serviceType) const {
    std::map<std::string, BaseService*>::const_iterator itr = m_ServiceList.find(_serviceType);
    if (itr != m_ServiceList.end()) {
        return itr->second;
    } else {
        return NULL;
    }
}

Group::ConstIterator Group::Begin() const {
    return m_ServiceList.begin();
}

Group::ConstIterator Group::End() const {
    return m_ServiceList.end();
}
