#include "ipdevice/Property.h"

using namespace IPDevice;
using namespace Utility;

const std::string Property::m_TypeNames[] = {"number", "text", "bool", "range", "attribute", "color", "changeble_attribute"};
const unsigned int Property::MAX_OPTION_SIZE = 15;

Property::Property(const std::string &_name, Jzon::Node _initialValue, bool _readOnly, bool _used, PropertyType _type, ServicePropertyDelegate* _function) :
    m_Name(_name),
    m_ReadOnly(_readOnly),
    m_Used(_used),
    m_Type(_type),
    m_Function(_function) {

    m_Value = _initialValue;
    SetTimestamp();
}

std::string Property::GetName() const {
    return m_Name;
}

Jzon::Node Property::GetValue() {
    ScopedLock lock(m_Mutex);
    return m_Value;
}

bool Property::IsReadOnly() const {
    return m_ReadOnly;
}

bool Property::IsUsed() const {
    return m_Used;
}

std::string Property::GetStringType() const {
    return m_TypeNames[m_Type];
}

Property::PropertyType Property::GetType() const {
    return m_Type;
}

const std::vector<Jzon::Node>& Property::GetOptions() const {
    return m_Options;
}

ServicePropertyDelegate::Ptr Property::GetPropertyFunction() const {
    return m_Function;
}

unsigned long long Property::GetTimestamp() const {
    return m_Timestamp;
}

void Property::SetName(const std::string &_name) {
    m_Name = _name;
}

void Property::SetValue(Jzon::Node _value) {
    ScopedLock lock(m_Mutex);
    m_Value = _value;
    SetTimestamp();
}

void Property::SetReadOnly(bool _readOnly) {
    m_ReadOnly = _readOnly;
}

void Property::SetUsed(bool _used) {
    m_Used = _used;
}

void Property::SetOptions(const std::vector<Jzon::Node>& _options) {
    if (_options.size() > MAX_OPTION_SIZE) {
        m_Options.assign(_options.begin(), _options.begin() + MAX_OPTION_SIZE - 1);
    } else {
        m_Options = _options;
    }
}

void Property::SetPropertyFunction(ServicePropertyDelegate::Ptr _function) {
    m_Function = _function;
}

void Property::SetType(PropertyType _type) {
    m_Type = _type;
}

void Property::SetTimestamp() {
    struct timeval timeVal;
    gettimeofday(&timeVal, NULL);
    m_Timestamp = timeVal.tv_sec * 1000ULL + timeVal.tv_usec / 1000ULL;
}

Property& Property::AddOption(Jzon::Node _value) {
    m_Options.push_back(_value);
    return *this;
}

void Property::ToJson(JsonStruct& _json, bool _stripped) const {
    _json.Insert("name", m_Name);
    _json.Insert("value", m_Value);
    if (!_stripped) {
        _json.Insert("readOnly", m_ReadOnly);
        _json.Insert("type", (int) m_Type);
        _json.Insert("options", m_Options);
    }
}

