#include "ipdevice/Service.h"

using namespace IPDevice;
using namespace Utility;

BaseService::BaseService(const std::string &_type) : m_Type(_type) {
}

std::string BaseService::GetServiceType() const {
    return m_Type;
}

void BaseService::AddProperty(const Property& _property) {
    m_PropertyList[_property.GetName()] = _property;
}

void BaseService::AddCommand(const Command& _command) {
    m_CommandList[_command.GetName()] = _command;
}

Property& BaseService::FindPropertyByName(const std::string &_name) {
    std::map<std::string, Property>::iterator itr = m_PropertyList.find(_name);
    if (itr != m_PropertyList.end()) {
        return itr->second;
    } else {
        throw NotFoundException("Property does not exist");
    }
}

Command& BaseService::FindCommandByName(const std::string& _name) {
    std::map<std::string, Command>::iterator itr = m_CommandList.find(_name);
    if (itr != m_CommandList.end()) {
        return itr->second;
    } else {
        throw NotFoundException("Command does not exist");
    }
}

void BaseService::SetPropertyFunction(const std::string &_property, ServicePropertyDelegate::Ptr _function) {
    std::map<std::string, Property>::iterator itr = m_PropertyList.find(_property);
    if (itr != m_PropertyList.end()) {
        itr->second.SetPropertyFunction(_function);
    } else {
        throw NotFoundException("Property does not exist");
    }
}

void BaseService::SetCommandFunction(const std::string &_command, ServiceCommandDelegate::Ptr _function) {
    std::map<std::string, Command>::iterator itr = m_CommandList.find(_command);
    if (itr != m_CommandList.end()) {
        itr->second.SetCommandFunction(_function);
    } else {
        throw NotFoundException("Command does not exist");
    }
}

void BaseService::ToJson(Utility::JsonStruct &_json, unsigned long long _timestamp, bool _stripped) const {
    std::vector<Jzon::Node> propertyArray;
    PropertyIterator itr = m_PropertyList.begin();
    for (; itr != m_PropertyList.end(); ++itr) {
        JsonStruct json;
        if (_timestamp == 0) {
            itr->second.ToJson(json, _stripped);
            propertyArray.push_back(json.GetNode());
        } else if (_timestamp < itr->second.GetTimestamp()) {
            itr->second.ToJson(json, _stripped);
            propertyArray.push_back(json.GetNode());
        }
    }
    if (!propertyArray.empty()) {
        _json.Insert("type", m_Type);
        _json.Insert("propertyList", propertyArray);
    }
}

BaseService::PropertyIterator BaseService::BeginProperty() const {
    return m_PropertyList.begin();
}

BaseService::PropertyIterator BaseService::EndProperty() const {
    return m_PropertyList.end();
}

BaseService::CommandIterator BaseService::BeginCommand() const {
    return m_CommandList.begin();
}

BaseService::CommandIterator BaseService::EndCommand() const {
    return m_CommandList.end();
}
