#include "ipdevice/SleepingDevice.h"

using namespace IPDevice;
using namespace Utility;

SleepingDevice::SleepingDevice(long _sleepingTimeout) : Device(), m_SleepingTimeout(_sleepingTimeout) {
    m_Timer = new Timer(ALIVE_TIME, false, -1);
}

SleepingDevice::~SleepingDevice() {
    m_Timer->StopTimer();
    SAFE_DELETE_O(m_Timer);
}

void SleepingDevice::Start() {
    m_State = ALIVE;
    m_Timer->StartTimer(Callback<SleepingDevice>(*this, &SleepingDevice::OnStateChange));
}

void SleepingDevice::Stop() {
    m_State = ALIVE;
    m_Timer->StopTimer();
}

void SleepingDevice::GetState(const Messenger::RequestMessage::Ptr _req, Messenger::ResponseMessage::Ptr _resp) {
    ScopedLock lock(m_Mutex);
    m_Timer->RestartTimer(ALIVE_TIME, false, -1);
    Device::GetState(_req, _resp);
}

void SleepingDevice::GetDeviceInfo(const Messenger::RequestMessage::Ptr _req, Messenger::ResponseMessage::Ptr _resp) {
    ScopedLock lock(m_Mutex);
    m_Timer->RestartTimer(ALIVE_TIME, false, -1);
    Device::GetDeviceInfo(_req, _resp);
}

void SleepingDevice::SetTime(const Messenger::RequestMessage::Ptr _req, Messenger::ResponseMessage::Ptr _resp) {
    ScopedLock lock(m_Mutex);
    m_Timer->RestartTimer(ALIVE_TIME, false, -1);
    Device::SetTime(_req, _resp);
}

void SleepingDevice::AdjustTime(const Messenger::RequestMessage::Ptr _req, Messenger::ResponseMessage::Ptr _resp) {
    ScopedLock lock(m_Mutex);
    m_Timer->RestartTimer(ALIVE_TIME, false, -1);
    Device::AdjustTime(_req, _resp);
}

void SleepingDevice::VerifyIdentity(const Messenger::RequestMessage::Ptr _req, Messenger::ResponseMessage::Ptr _resp) {
    ScopedLock lock(m_Mutex);
    m_Timer->RestartTimer(ALIVE_TIME, false, -1);
    Device::VerifyIdentity(_req, _resp);
}

void SleepingDevice::GetGroupList(const Messenger::RequestMessage::Ptr _req, Messenger::ResponseMessage::Ptr _resp) {
    ScopedLock lock(m_Mutex);
    m_Timer->RestartTimer(ALIVE_TIME, false, -1);
    Device::GetGroupList(_req, _resp);
}

void SleepingDevice::GetServiceList(const Messenger::RequestMessage::Ptr _req, Messenger::ResponseMessage::Ptr _resp) {
    ScopedLock lock(m_Mutex);
    m_Timer->RestartTimer(ALIVE_TIME, false, -1);
    Device::GetServiceList(_req, _resp);
}

void SleepingDevice::GetService(const Messenger::RequestMessage::Ptr _req, Messenger::ResponseMessage::Ptr _resp) {
    ScopedLock lock(m_Mutex);
    m_Timer->RestartTimer(ALIVE_TIME, false, -1);
    Device::GetService(_req, _resp);
}

void SleepingDevice::GetPropertyList(unsigned int _groupId, const std::string &_serviceType, const Messenger::RequestMessage::Ptr _req, Messenger::ResponseMessage::Ptr _resp) {
    ScopedLock lock(m_Mutex);
    m_Timer->RestartTimer(ALIVE_TIME, false, -1);
    Device::GetPropertyList(_groupId, _serviceType, _req, _resp);
}

void SleepingDevice::SetPropertyValue(unsigned int _groupId, const std::string &_serviceType, const Messenger::RequestMessage::Ptr _req, Messenger::ResponseMessage::Ptr _resp) {
    ScopedLock lock(m_Mutex);
    m_Timer->RestartTimer(ALIVE_TIME, false, -1);
    Device::SetPropertyValue(_groupId, _serviceType, _req, _resp);
}

void SleepingDevice::GetPropertyValue(unsigned int _groupId, const std::string &_serviceType, const Messenger::RequestMessage::Ptr _req, Messenger::ResponseMessage::Ptr _resp) {
    ScopedLock lock(m_Mutex);
    m_Timer->RestartTimer(ALIVE_TIME, false, -1);
    Device::GetPropertyValue(_groupId, _serviceType, _req, _resp);
}

void SleepingDevice::GetCommandList(unsigned int _groupId, const std::string &_serviceType, const Messenger::RequestMessage::Ptr _req, Messenger::ResponseMessage::Ptr _resp) {
    ScopedLock lock(m_Mutex);
    m_Timer->RestartTimer(ALIVE_TIME, false, -1);
    Device::GetCommandList(_groupId, _serviceType, _req, _resp);
}

void SleepingDevice::ExecuteCommand(unsigned int _groupId, const std::string &_serviceType, const Messenger::RequestMessage::Ptr _req, Messenger::ResponseMessage::Ptr _resp) {
    ScopedLock lock(m_Mutex);
    m_Timer->RestartTimer(ALIVE_TIME, false, -1);
    Device::ExecuteCommand(_groupId, _serviceType, _req, _resp);
}

void SleepingDevice::SetPropertyValueIntern(unsigned int _groupId, const std::string &_serviceType, const std::string &_name, const Jzon::Node& _value, bool _sendEvent) {
    ScopedLock lock(m_Mutex);
    if (m_State == SLEEP) {
        try {
            StartReconnection();
        } catch (const ProtocolException& exc) {
            return;
        }

        m_State = ALIVE;
    }
    m_Timer->SkipInvocation(true);
    m_Timer->RestartTimer(ALIVE_TIME, false, -1);
    Device::SetPropertyValueIntern(_groupId, _serviceType, _name, _value, _sendEvent);
}

Jzon::Node SleepingDevice::GetPropertyValueIntern(unsigned int _groupId, const std::string &_serviceType, const std::string &_name) {
    return Device::GetPropertyValueIntern(_groupId, _serviceType, _name);
}

bool SleepingDevice::UpdatePropertyAttributeList(unsigned int _groupId, const std::string& _serviceType, const std::string& _name, const std::vector<Jzon::Node>& _attributeList) {
    return Device::UpdatePropertyAttributeList(_groupId, _serviceType, _name, _attributeList);
}

void SleepingDevice::OnStateChange(Timer &_timer) {
    ScopedLock lock(m_Mutex);
    if (!_timer.ShouldSkip()) {
        if (m_State == ALIVE) {
            DisconnectWithoutStsMessage();
            Destroy();
            _timer.RestartTimer(m_SleepingTimeout, false, -1);
            m_State = SLEEP;
            LOG(Debug, "State changed to sleep");
        } else {
            try {
                StartReconnection();
            } catch (const ProtocolException& exc) {
                return;
            }
            _timer.RestartTimer(ALIVE_TIME, false, -1);
            m_State = ALIVE;
            LOG(Debug, "State changed to alive");
        }
    }
}
