#include "messenger/MQTTLogger.h"

namespace Messenger {

MQTTLogger::MQTTLogger() {
    Log_setTraceCallback(&MQTTLogger::LogCallback);
    Log_setTraceLevel(TRACE_MAXIMUM);
}

MQTTLogger::~MQTTLogger() {
}

void MQTTLogger::SetListener(Utility::ILogListener* _listener) {
    m_Listener = _listener;
}

MQTTLogger& MQTTLogger::GetInstance() {
    static MQTTLogger instance;
    return instance;
}

void MQTTLogger::LogCallback(enum LOG_LEVELS _level, const char* _message) {
    std::string message = _message != NULL ? _message : "";

    if (message.length() > 20 && _message[8]==' '
            && _message[15]=='.' && _message[19]==' ' ) {
        // strip date and time from the begining of the message.
        // Format: '20181214 080018.933 ' = 20 characters
        message = message.substr(20);
    }

    switch (_level) {
        case INVALID_LEVEL:
        case TRACE_MAXIMUM:
            break;
        case TRACE_MEDIUM:
            MQTTLogger::GetInstance().OnTrace(message);
            break;
        case TRACE_MINIMUM:
            MQTTLogger::GetInstance().OnDebug(message);
            break;
        case TRACE_PROTOCOL:
            MQTTLogger::GetInstance().OnInfo(message);
            break;
        case LOG_ERROR:
            MQTTLogger::GetInstance().OnError(message);
            break;
        case LOG_FATAL:
            MQTTLogger::GetInstance().OnFatal(message);
            break;
        case LOG_SEVERE:
            MQTTLogger::GetInstance().OnWarn(message);
            break;
        default:
            break;
    }
}

void MQTTLogger::OnTrace(const std::string &_msg) {
    if (m_Listener != NULL) {
        m_Listener->OnTrace(_msg);
    }
}

void MQTTLogger::OnDebug(const std::string &_msg) {
    if (m_Listener != NULL) {
        m_Listener->OnDebug(_msg);
    }
}

void MQTTLogger::OnInfo(const std::string &_msg) {
    if (m_Listener != NULL) {
        m_Listener->OnInfo(_msg);
    }
}

void MQTTLogger::OnWarn(const std::string &_msg) {
    if (m_Listener != NULL) {
        m_Listener->OnWarn(_msg);
    }
}

void MQTTLogger::OnError(const std::string &_msg) {
    if (m_Listener != NULL) {
        m_Listener->OnError(_msg);
    }
}

void MQTTLogger::OnFatal(const std::string &_msg) {
    if (m_Listener != NULL) {
        m_Listener->OnFatal(_msg);
    }
}

}
