#include "messenger/MQTTMessage.h"

using namespace Messenger;
using namespace Utility;

const MQTTClient_message& InitPahoMsg() {
    static MQTTClient_message initMsg = MQTTClient_message_initializer;
    return initMsg;
}

Message::Message() : m_QoS(QOS0), m_Retained(false), m_Duplicated(false) {
}

Message::Message(const std::string &_payload) : m_Payload(_payload), m_QoS(QOS0), m_Retained(false), m_Duplicated(false) {
}

Message::Message(const Message &_msg) : m_Payload(_msg.m_Payload), m_QoS(_msg.m_QoS), m_Retained(_msg.m_Retained), m_Duplicated(_msg.m_Duplicated) {
}

Message::~Message() {
}

std::string Message::GetPayload() const {
    return m_Payload;
}

void Message::SetPayload(const std::string &_payload) {
    m_Payload = _payload;
}

Message::QosLevel Message::GetQoS() const {
    return m_QoS;
}

void Message::SetQoS(QosLevel _qos) {
    m_QoS = _qos;
}

bool Message::IsRetained() const {
    return m_Retained;
}

void Message::SetRetained(bool _ret) {
    m_Retained = _ret;
}

bool Message::IsDuplicated() const {
    return m_Duplicated;
}

void Message::SetDuplicated(bool _dup) {
    m_Duplicated = _dup;
}

void Message::WriteMessage(MQTTClient_message **_msg) const {
    *_msg = (MQTTClient_message* ) malloc(sizeof(MQTTClient_message));
    **_msg = InitPahoMsg();
    char* payload = strndup(m_Payload.c_str(), m_Payload.size());
    (*_msg)->payloadlen = m_Payload.size();
    (*_msg)->payload = payload;
    (*_msg)->qos = m_QoS;
    (*_msg)->retained = m_Retained ? 1 : 0;
    (*_msg)->dup = m_Duplicated ? 1 : 0;
}

void Message::FreeMessage(MQTTClient_message **_msg) {
    if (_msg != NULL) {
        MQTTClient_message* msg = *_msg;
        free(msg->payload);
        free(msg);
        msg = NULL;
    }
}

Message::Ptr Message::ParseMessage(const MQTTClient_message *_msg) {
    Message::Ptr dest = new Message();
    std::string payload((char*) _msg->payload, _msg->payloadlen);
    dest->SetPayload(payload);
    dest->SetQoS((QosLevel) _msg->qos);
    dest->SetRetained(_msg->retained);
    dest->SetDuplicated(_msg->dup);
    return dest;
}
