#include "messenger/NetworkProxy.h"

using namespace Messenger;
using namespace Utility;

const std::string ConnectionRunner::NO_LOST = "no_lost";

ConnectionRunner::ConnectionRunner(IConnListener &_connListener) : m_ConnListener(_connListener) {
    Reinitialize();
}

void ConnectionRunner::Reinitialize() {
    m_Cause = NO_LOST;
    SynchronizedRunner::Reinitialize();
}

void ConnectionRunner::Report(const std::string &_cause) {
    m_Cause = _cause;
    Continue();
}

void ConnectionRunner::DoJob() {
    if (m_Cause != NO_LOST) {
        m_ConnListener.OnConnectionLost(m_Cause);
        m_Cause = NO_LOST;
    } else {
        Pause();
    }
}

AsyncNetworkProxy::AsyncNetworkProxy(INetworkProxyListener &_listener) :
    m_Listener(_listener),
    m_SendingQueue(*this),
    m_RecvQueue(*this),
    m_ConnRunner(_listener),
    m_Stopped(false) {
    m_SendingThread.Start(m_SendingQueue);
    m_RecvThread.Start(m_RecvQueue);
    m_ConnThread.Start(m_ConnRunner);
}

AsyncNetworkProxy::~AsyncNetworkProxy() {
    m_Stopped = true;
    m_SendingQueue.Stop();
    m_RecvQueue.Stop();
    m_ConnRunner.Stop();
    m_SendingThread.Join();
    m_RecvThread.Join();
    m_ConnThread.Join();
}

void AsyncNetworkProxy::Publish(const std::string &_topic, const Message::Ptr _msg) {
    if (!m_Stopped) {
        PublishHolder::Ptr holder = new PublishHolder(_topic, _msg);
        m_SendingQueue.Add(holder);
    }
}

void AsyncNetworkProxy::ConnectionLost(const std::string &_cause) {
    if (!m_Stopped) {
        m_ConnRunner.Report(_cause);
    }
}

void AsyncNetworkProxy::MessageArrived(const std::string &_topic, const Message::Ptr _msg) {
    if (!m_Stopped) {
        RecvHolder::Ptr holder = new RecvHolder(_topic, _msg);
        m_RecvQueue.Add(holder);
    }
}

void AsyncNetworkProxy::DeliveryComplete(const IDeliveryToken &_token) {
}

void AsyncNetworkProxy::PublishFailed(int _code, const std::string &_topic, const Message::Ptr _msg) {
    if (!m_Stopped) {
        m_Listener.OnPublishFailed(_code, _topic, _msg);
    }
}

void AsyncNetworkProxy::OnProcess(PublishHolder::Ptr _object) {
    if (!m_Stopped) {
        std::string topic = _object->GetTopic();
        Message::Ptr msg = _object->GetMessage();
        m_Listener.OnPublishRequired(topic, msg);
    }
}

void AsyncNetworkProxy::OnProcess(RecvHolder::Ptr _object) {
    if (!m_Stopped) {
        std::string topic = _object->GetTopic();
        Message::Ptr msg = _object->GetMessage();
        m_Listener.OnMessageArrived(topic, msg);
    }
}

