#include "messenger/SynchronizedRunner.h"

using namespace Messenger;
using namespace Utility;

SynchronizedRunner::SynchronizedRunner() {
    Reinitialize();
}

SynchronizedRunner::~SynchronizedRunner() {
    Stop();
}

void SynchronizedRunner::Reinitialize() {
    m_Running = true;
}

void SynchronizedRunner::Stop() {
    Lock();
    m_Running = false;
    Unlock();
    Continue();
}

void SynchronizedRunner::Pause() {
    if (IsRunning()) {
        m_Sync.Wait();
    }
}

void SynchronizedRunner::Continue() {
    m_Sync.Notify();
}

void SynchronizedRunner::Lock() {
    m_Mutex.Lock();
}

void SynchronizedRunner::Unlock() {
    m_Mutex.Unlock();
}

bool SynchronizedRunner::IsRunning() const {
    return m_Running;
}

void SynchronizedRunner::Run() {
    while (IsRunning()) {
        DoJob();
    }
}
