#include "util/Event.h"

using namespace Utility;

Event::Event() : m_State(false) {
    if (pthread_mutex_init(&m_Mutex, NULL)) {
        /* TODO: Error handling */
        throw BaseException("Mutex init failed");
    }

    pthread_condattr_t attr;
    if (pthread_condattr_init(&attr))
    {
        pthread_mutex_destroy(&m_Mutex);
        /* TODO: Error handling */
        throw BaseException("Attributes init failed");
    }
    if (pthread_condattr_setclock(&attr, CLOCK_MONOTONIC))
    {
        pthread_condattr_destroy(&attr);
        pthread_mutex_destroy(&m_Mutex);
        /* TODO: Error handling */
        throw BaseException("Setting clock failed");
    }
    if (pthread_cond_init(&m_Cond, &attr))
    {
        pthread_condattr_destroy(&attr);
        pthread_mutex_destroy(&m_Mutex);
        /* TODO: Error handling */
        throw BaseException("Condition variable init failed");
    }
    pthread_condattr_destroy(&attr);
}

Event::~Event() {
    pthread_cond_destroy(&m_Cond);
    pthread_mutex_destroy(&m_Mutex);
}

void Event::Wait() {
    if (pthread_mutex_lock(&m_Mutex)) {
        /* TODO: Error handling */
        throw BaseException("Mutex lock failed");
    }
    while(!m_State) {
        if (pthread_cond_wait(&m_Cond, &m_Mutex)) {
            pthread_mutex_unlock(&m_Mutex);
            /* TODO: Error handling */
            throw BaseException("Cannot wait");
        }
    }
    m_State = false;
    pthread_mutex_unlock(&m_Mutex);
}

bool Event::Wait(long _milliseconds) {
    struct timespec timeSpec;
    clock_gettime(CLOCK_MONOTONIC, &timeSpec);

    timeSpec.tv_sec += _milliseconds / 1000;
    timeSpec.tv_nsec += (_milliseconds % 1000) * 1000000;
    if (timeSpec.tv_nsec >= 1000000000)
    {
        timeSpec.tv_nsec -= 1000000000;
        timeSpec.tv_sec++;
    }

    int rc = pthread_mutex_lock(&m_Mutex);

    while (!m_State) {
        if ((rc = pthread_cond_timedwait(&m_Cond, &m_Mutex, &timeSpec))) {
            if (rc == ETIMEDOUT)
                break;
            pthread_mutex_unlock(&m_Mutex);
        }
    }
    if (rc == 0)
        m_State = false;
    pthread_mutex_unlock(&m_Mutex);
    if (rc == ETIMEDOUT) {
        throw TimeoutException("Waiting timeout");
    }
    return rc == 0;
}

void Event::Set() {
    if (pthread_mutex_lock(&m_Mutex)) {
        /* TODO: Error handling */
        throw BaseException("Mutex lock failed");
    }
    m_State = true;
    if (pthread_cond_broadcast(&m_Cond))
    {
        pthread_mutex_unlock(&m_Mutex);
        /* TODO: Error handling */
        throw BaseException("Notify failed");
    }
    pthread_mutex_unlock(&m_Mutex);
}

void Event::Reset() {
    if (pthread_mutex_lock(&m_Mutex)) {
        /* TODO: Error handling */
        throw BaseException("Mutex lock failed");
    }
    m_State = false;
    pthread_mutex_unlock(&m_Mutex);
}

