#include "util/HTTPClient.h"

using namespace Utility;

HTTPClient::HTTPClient(const std::string _url) : m_URL(_url) {
    m_Handle = curl_easy_init();
    if (m_Handle == NULL) {
        throw BaseException("Failed to create handle");
    }
}

HTTPClient::~HTTPClient(){
    curl_easy_cleanup(m_Handle);
}

std::string HTTPClient::Fetch() {
    SetOptions();
    SendGetRequest();
    return m_Data;
}

void HTTPClient::SetOptions() {
    CURLcode res;

    res = curl_easy_setopt(m_Handle, CURLOPT_URL, m_URL.c_str());
    if (res != CURLE_OK) {
        throw BaseException("Setting option failed", res);
    }

    res = curl_easy_setopt(m_Handle, CURLOPT_NOPROGRESS, 1L);
    if (res != CURLE_OK) {
        throw BaseException("Setting option failed", res);
    }

    res = curl_easy_setopt(m_Handle, CURLOPT_WRITEFUNCTION, WriteDataCallback);
    if (res != CURLE_OK) {
        throw BaseException("Setting option failed", res);
    }

    res = curl_easy_setopt(m_Handle, CURLOPT_WRITEDATA, this);
    if (res != CURLE_OK) {
        throw BaseException("Setting option failed", res);
    }
}

void HTTPClient::SendGetRequest() {
    CURLcode res;
    res = curl_easy_perform(m_Handle);

    if (res != CURLE_OK) {
        throw BaseException("Sending get request failed", res);
    }
}

size_t HTTPClient::WriteDataCallback(void *_buffer, size_t _size, size_t _nmemb, void *_userData) {
    return (static_cast<HTTPClient*>(_userData))->WriteData(_buffer, _size, _nmemb);
}

size_t HTTPClient::WriteData(void *_buffer, size_t _size, size_t _nmemb) {
    size_t numOfBytes = _size * _nmemb;

    m_Data += std::string((char*) _buffer);

    return numOfBytes;
}

