#include "util/Mutex.h"

using namespace Utility;

Mutex::Mutex() {
    pthread_mutexattr_t attr;
    pthread_mutexattr_init(&attr);
    pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE);

    if (pthread_mutex_init(&m_Mutex, &attr)) {
        pthread_mutexattr_destroy(&attr);
        /* TODO: Error handling */
        return;
    }
    pthread_mutexattr_destroy(&attr);
}

Mutex::~Mutex() {
    pthread_mutex_destroy(&m_Mutex);
}

void Mutex::Lock() {
    if (pthread_mutex_lock(&m_Mutex)) {
        /* TODO: Error handling */
        std::cout << "Error locking" << std::endl;
        return;
    }
}

bool Mutex::TryLock() {
    int rc = pthread_mutex_trylock(&m_Mutex);

    if (rc == 0)
        return true;
    else
        return false;
}

bool Mutex::TryLock(long milliseconds) {
    struct timespec timeSpec;
    clock_gettime(CLOCK_MONOTONIC, &timeSpec);

    timeSpec.tv_sec += milliseconds / 1000;
    timeSpec.tv_nsec += (milliseconds % 1000) * 1000000;
    if (timeSpec.tv_nsec >= 1000000000)
    {
        timeSpec.tv_nsec -= 1000000000;
        timeSpec.tv_sec++;
    }

    int rc = pthread_mutex_timedlock(&m_Mutex, &timeSpec);
    if (rc == 0) {
        return true;
    } else {
        /* TODO: Error handling */
        return false;
    }
}

void Mutex::Unlock() {
    if (pthread_mutex_unlock(&m_Mutex)) {
        /* TODO: Error handling */
        std::cout << "Error unlocking" << std::endl;
        return;
    }
}
