#include "util/Timer.h"

using namespace Utility;

Timer::Timer(long _timerInterval,  bool _nowStart, int _repeatTimes) : m_TimerInterval(_timerInterval), m_NowStart(_nowStart), m_RepeatTimes(_repeatTimes), m_Skip(false), m_Callback(0) {
    m_Terminated = true;
    m_ShouldRun = false;
}

Timer::~Timer() {
    if (!m_Terminated) {
        StopTimer();
    }
}

void Timer::StartTimer(const AbstractCallback &_method) {
    ScopedLock lock(m_Mutex);
    if (m_Terminated) {
        m_ShouldRun = true;
        m_Terminated = false;
        if (!m_Callback) {
            m_Callback = _method.Clone();
        }

        m_Thread.Start(*this);
    }
}

void Timer::StopTimer() {
    ScopedLock lock(m_Mutex);
    if (!m_Terminated) {
        m_ShouldRun = false;

        m_Sync.Notify();
        if (m_Callback) {
            delete m_Callback;
            m_Callback = 0;
        }
        m_Thread.Join();
        m_Terminated = true;
    }
}

void Timer::RestartTimer(long _timerInterval, bool _nowStart, int _repeatTimes) {
    ScopedLock lock(m_Mutex);
    if (!m_Terminated) {
        if (_timerInterval == 0) {
            m_ShouldRun = false;
            m_Terminated = true;
        } else {
            m_TimerInterval = _timerInterval;
            m_NowStart = _nowStart;
            m_RepeatTimes = _repeatTimes;
        }
        m_Sync.Notify();
    }
}

void Timer::SkipInvocation(bool _skip) {
    ScopedLock lock(m_Mutex);
    m_Skip = _skip;
}

bool Timer::ShouldSkip() {
    ScopedLock lock(m_Mutex);
    return m_Skip;
}

void Timer::SetTimeout(long _timerInterval) {
    ScopedLock lock(m_Mutex);
    m_TimerInterval = _timerInterval;
}

void Timer::SetRepeatTimes(int _repeatTimes) {
    ScopedLock lock(m_Mutex);
    m_RepeatTimes = _repeatTimes;
}

void Timer::Run() {
    while (m_ShouldRun) {
        if (m_NowStart) {
            if (m_RepeatTimes > 0) {
                m_Callback->Invoke(*this);
                m_RepeatTimes--;
            } else if (m_RepeatTimes < 0) {
                m_Callback->Invoke(*this);
            } else {
                m_ShouldRun = false;
            }
        } else {
            m_NowStart = true;
        }
        if (m_ShouldRun) {
            try {
                m_Skip = false;
                m_Sync.Wait(m_TimerInterval);
            } catch (const BaseException& exc) {
            }
        } else {
            break;
        }
    }
}
